package de.fhdw.wtf.common.stream.test;

import static org.junit.Assert.assertEquals;

import org.junit.Test;

import de.fhdw.wtf.common.token.CommentTextToken;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.InvalidToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.keywords.AbstractToken;
import de.fhdw.wtf.common.token.keywords.ClassToken;
import de.fhdw.wtf.common.token.keywords.GroupToken;
import de.fhdw.wtf.common.token.keywords.PriorToken;
import de.fhdw.wtf.common.token.symbols.AsteriskToken;
import de.fhdw.wtf.common.token.symbols.ColonToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketOpenToken;
import de.fhdw.wtf.common.token.symbols.EqualToken;
import de.fhdw.wtf.common.token.symbols.GreaterSymbolToken;
import de.fhdw.wtf.common.token.symbols.PlusSymbolToken;
import de.fhdw.wtf.common.token.symbols.SemicolonToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketOpenToken;

public class LengthTest {
	
	@Test
	public void lengthSymbol() {
		assertEquals(1, SemicolonToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(1, AsteriskToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(1, CurlyBracketOpenToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(1, CurlyBracketCloseToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(1, ColonToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(1, EqualToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(1, GreaterSymbolToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(1, PlusSymbolToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(1, SquareBracketCloseToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(1, SquareBracketOpenToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
	}
	
	@Test
	public void lengthKeywords() {
		assertEquals(8, AbstractToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(5, ClassToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(5, GroupToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(5, PriorToken.create(Position.create("", 0 + 1, 0 + 1, 0)).getLength());
	}
	
	@Test
	public void lengthIdentifier() {
		assertEquals(0, IdentifierToken.create("", Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(1, IdentifierToken.create("A", Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(2, IdentifierToken.create("AB", Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(3, IdentifierToken.create("ABC", Position.create("", 0 + 1, 0 + 1, 0)).getLength());
	}
	
	@Test
	public void lengthInvalid() {
		assertEquals(0, InvalidToken.create("", Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(1, InvalidToken.create("A", Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(2, InvalidToken.create("AB", Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(3, InvalidToken.create("ABC", Position.create("", 0 + 1, 0 + 1, 0)).getLength());
	}
	
	@Test
	public void lengthComment() {
		assertEquals(0, CommentTextToken.create("", Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(1, CommentTextToken.create("A", Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(2, CommentTextToken.create("AB", Position.create("", 0 + 1, 0 + 1, 0)).getLength());
		assertEquals(3, CommentTextToken.create("ABC", Position.create("", 0 + 1, 0 + 1, 0)).getLength());
	}
	
}
