package de.fhdw.wtf.dsl.scanner.test;

import static org.junit.Assert.assertEquals;

import java.io.IOException;

import org.junit.Test;

import de.fhdw.wtf.common.stream.ScannerInputStream;
import de.fhdw.wtf.common.stream.SimpleTokenStream;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.token.CommentTextToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.testutil.TestUtil;

/**
 * Test cases for Scanner working on physical model files.
 * 
 */
public class ScannerFileTest {
	
	private static final String PATH_TO_TEST_FILE_DIR = "src/test/java/de/fhdw/wtf/dsl/scanner/test/files/";
	private static final String MODEL_SUFFIX = "model";
	
	@Test
	public void modelFileMultiLine() throws IOException {
		final ScannerInputStream input =
				TestUtil.readFileToScannerStream(PATH_TO_TEST_FILE_DIR + "MultiLineComment" + '.' + MODEL_SUFFIX);
		final TokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		assertEquals(CommentTextToken.create("#A" + "\r\n" + "B#", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
	}
	
	// TODO Asserts (Bug #768)
	public void modelFilePerson() throws IOException {
		final ScannerInputStream input =
				TestUtil.readFileToScannerStream(PATH_TO_TEST_FILE_DIR + "Person" + '.' + MODEL_SUFFIX);
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		output.printStreamContent();
	}
}
