package de.fhdw.wtf.parser.testWithScanner;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.util.Vector;

import org.junit.Before;
import org.junit.Test;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.AttributeModifier;
import de.fhdw.wtf.common.ast.AttributeModifierTransient;
import de.fhdw.wtf.common.ast.Constructor;
import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.OperationModifier;
import de.fhdw.wtf.common.ast.OperationModifierAbstract;
import de.fhdw.wtf.common.ast.QualifiedName;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.ast.type.ByNameState;
import de.fhdw.wtf.common.ast.type.ClassModifier;
import de.fhdw.wtf.common.ast.type.ClassModifierAbstract;
import de.fhdw.wtf.common.ast.type.ClassModifierService;
import de.fhdw.wtf.common.ast.type.ClassModifierTransient;
import de.fhdw.wtf.common.ast.type.ClassModifierVisitable;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.ExceptionClassType;
import de.fhdw.wtf.common.ast.type.ListType;
import de.fhdw.wtf.common.ast.type.MapType;
import de.fhdw.wtf.common.ast.type.ProductElementType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.RegularClassType;
import de.fhdw.wtf.common.ast.type.SumType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.type.TypeProxy;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.keywords.AbstractToken;
import de.fhdw.wtf.common.token.keywords.GroupToken;
import de.fhdw.wtf.common.token.keywords.ServiceToken;
import de.fhdw.wtf.common.token.keywords.TransientToken;
import de.fhdw.wtf.common.token.keywords.VisitableToken;
import de.fhdw.wtf.common.token.symbols.BracketOpenToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketOpenToken;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;

/**
 * Test the integration of scanner and parser.
 */
public class TestParserScanner {
	
	private CurlyBracketOpenToken curlyBracketOpenToken;
	private BracketOpenToken bracketOpenToken;
	private AbstractToken abstractToken;
	
	/**
	 * Creates the most important Tokens.
	 * 
	 * @throws Exception
	 */
	@Before
	public void setUp() throws Exception {
		this.bracketOpenToken = BracketOpenToken.create(Position.create("", 1, 1, 0));
		this.curlyBracketOpenToken = CurlyBracketOpenToken.create(Position.create("", 1, 1, 0));
		this.abstractToken = AbstractToken.create(Position.create("", 1, 1, 0));
	}
	
	/**
	 * group : group = [ class : class = {op : [()->test];} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSimpleClassOperation() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken test = IdentifierToken.create("test", Position.create("", 1, 1, 0));
		final IdentifierToken op = IdentifierToken.create("op", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput("group1 : group = [ class1 : class = {op : [[()->test]];} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						new Vector<Attribute>(),
						new Vector<Type>(),
						operations,
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ProductType product = ProductType.create(this.bracketOpenToken);
		final ByNameState testByName = ByNameState.create(UnqualifiedName.create(test));
		final TypeProxy testTypeProxy = TypeProxy.create(test, testByName);
		
		final Operation operation =
				Operation.create("op", new Vector<OperationModifier>(), product, clazz, testTypeProxy, op);
		operations.add(operation);
		
		modifiers.add(ClassModifierService.create(ServiceToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(TransientToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {op : [(param:Integer)->test];} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSimpleClassOperation2() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken test = IdentifierToken.create("test", Position.create("", 1, 1, 0));
		final IdentifierToken op = IdentifierToken.create("op", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken paramIdentifier = IdentifierToken.create("param", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {op : [[(param:Integer)->test]];} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						new Vector<Attribute>(),
						new Vector<Type>(),
						operations,
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ProductType product = ProductType.create(this.bracketOpenToken);
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(paramIdentifier, integerByName);
		final ProductElementType productElement = ProductElementType.create("param", typeProxyInteger, paramIdentifier);
		product.addElement(productElement);
		
		final ByNameState testByName = ByNameState.create(UnqualifiedName.create(test));
		final TypeProxy testTypeProxy = TypeProxy.create(test, testByName);
		
		final Operation operation =
				Operation.create("op", new Vector<OperationModifier>(), product, clazz, testTypeProxy, op);
		operations.add(operation);
		
		modifiers.add(ClassModifierService.create(ServiceToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(TransientToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {op : [(param1:Integer, param2:{Integer,String})->test] abstract;} service
	 * transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSimpleClassOperation3() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken test = IdentifierToken.create("test", Position.create("", 1, 1, 0));
		final IdentifierToken op = IdentifierToken.create("op", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken param1Identifier = IdentifierToken.create("param1", Position.create("", 1, 1, 0));
		final IdentifierToken param2Identifier = IdentifierToken.create("param2", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {op : [[(param1:Integer, param2:{Integer,String})->test]] abstract;} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						new Vector<Attribute>(),
						new Vector<Type>(),
						operations,
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ProductType product = ProductType.create(this.bracketOpenToken);
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(param1Identifier, integerByName);
		final ProductElementType productElement1 =
				ProductElementType.create("param1", typeProxyInteger, param1Identifier);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(param2Identifier, stringByName);
		
		final SumType sum = SumType.create(this.curlyBracketOpenToken);
		sum.add(typeProxyInteger);
		sum.add(typeProxyString);
		
		final ProductElementType productElement2 = ProductElementType.create("param2", sum, param2Identifier);
		
		product.addElement(productElement1);
		product.addElement(productElement2);
		
		final ByNameState testByName = ByNameState.create(UnqualifiedName.create(test));
		final TypeProxy testTypeProxy = TypeProxy.create(test, testByName);
		
		final Vector<OperationModifier> operationModifiers = new Vector<>();
		operationModifiers.add(OperationModifierAbstract.create(this.abstractToken));
		
		final Operation operation = Operation.create("op", operationModifiers, product, clazz, testTypeProxy, op);
		operations.add(operation);
		
		modifiers.add(ClassModifierService.create(ServiceToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(TransientToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {op1 : [(param1:Integer, param2:{Integer,String})->test] abstract; op2 :
	 * [(param1:Integer, param2:{Integer,String})->test] abstract;} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSimpleClassOperation4() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken test = IdentifierToken.create("test", Position.create("", 1, 1, 0));
		final IdentifierToken op1 = IdentifierToken.create("op1", Position.create("", 1, 1, 0));
		final IdentifierToken op2 = IdentifierToken.create("op2", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken param1Identifier = IdentifierToken.create("param1", Position.create("", 1, 1, 0));
		final IdentifierToken param2Identifier = IdentifierToken.create("param2", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {op1 : [[(param1:Integer, param2:{Integer,String})->test]] abstract; op2 : [[(param1:Integer, param2:{Integer,String})->test]] abstract;} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						new Vector<Attribute>(),
						new Vector<Type>(),
						operations,
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ProductType product = ProductType.create(this.bracketOpenToken);
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(param1Identifier, integerByName);
		final ProductElementType productElement1 =
				ProductElementType.create("param1", typeProxyInteger, param1Identifier);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(param2Identifier, stringByName);
		
		final SumType sum = SumType.create(this.curlyBracketOpenToken);
		sum.add(typeProxyInteger);
		sum.add(typeProxyString);
		
		final ProductElementType productElement2 = ProductElementType.create("param2", sum, param2Identifier);
		
		product.addElement(productElement1);
		product.addElement(productElement2);
		
		final ByNameState testByName = ByNameState.create(UnqualifiedName.create(test));
		final TypeProxy testTypeProxy = TypeProxy.create(test, testByName);
		
		final Vector<OperationModifier> operationModifiers = new Vector<>();
		operationModifiers.add(OperationModifierAbstract.create(this.abstractToken));
		
		final Operation operation1 = Operation.create("op1", operationModifiers, product, clazz, testTypeProxy, op1);
		final Operation operation2 = Operation.create("op2", operationModifiers, product, clazz, testTypeProxy, op2);
		operations.add(operation1);
		operations.add(operation2);
		
		modifiers.add(ClassModifierService.create(ServiceToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(TransientToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {op1 : [(param1:Integer, param2:{})->test] abstract; op2 : [(param1:Integer,
	 * param2:{Integer,String})->test] abstract;} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSimpleClassOperation5Empty() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken test = IdentifierToken.create("test", Position.create("", 1, 1, 0));
		final IdentifierToken op1 = IdentifierToken.create("op1", Position.create("", 1, 1, 0));
		final IdentifierToken op2 = IdentifierToken.create("op2", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken param1Identifier = IdentifierToken.create("param1", Position.create("", 1, 1, 0));
		final IdentifierToken param2Identifier = IdentifierToken.create("param2", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {op1 : [[(param1:Integer, param2:{})->test]] abstract; op2 : [[(param1:Integer, param2:{Integer,String})->test]] abstract;} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Operation> operations = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						new Vector<Attribute>(),
						new Vector<Type>(),
						operations,
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(param1Identifier, integerByName);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(param2Identifier, stringByName);
		
		final ByNameState testByName = ByNameState.create(UnqualifiedName.create(test));
		final TypeProxy testTypeProxy = TypeProxy.create(test, testByName);
		
		final ProductType product1 = ProductType.create(this.bracketOpenToken);
		final ProductElementType productElement1 =
				ProductElementType.create("param1", typeProxyInteger, param1Identifier);
		final SumType sum = SumType.create(this.curlyBracketOpenToken);
		final ProductElementType productElement2 = ProductElementType.create("param2", sum, param2Identifier);
		product1.addElement(productElement1);
		product1.addElement(productElement2);
		
		final Vector<OperationModifier> operationModifiers = new Vector<>();
		operationModifiers.add(OperationModifierAbstract.create(this.abstractToken));
		
		final Operation operation1 = Operation.create("op1", operationModifiers, product1, clazz, testTypeProxy, op1);
		
		final ProductType product2 = ProductType.create(this.bracketOpenToken);
		product2.addElement(productElement1);
		
		final SumType sum2 = SumType.create(this.curlyBracketOpenToken);
		sum2.add(typeProxyInteger);
		sum2.add(typeProxyString);
		
		final ProductElementType productElement3 = ProductElementType.create("param2", sum2, param2Identifier);
		product2.addElement(productElement3);
		final Operation operation2 = Operation.create("op2", operationModifiers, product2, clazz, testTypeProxy, op2);
		operations.add(operation1);
		operations.add(operation2);
		
		modifiers.add(ClassModifierService.create(ServiceToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(TransientToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {map : [String->Integer];} visitable abstract;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testModifierVisitable() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken mapIdentifier = IdentifierToken.create("map", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {map : [String->Integer];} visitable abstract;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(integerIdentifier, integerByName);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(stringIdentifier, stringByName);
		final MapType map = MapType.create(mapIdentifier, typeProxyString, typeProxyInteger);
		
		final Attribute attribute = Attribute.create("map", map, new Vector<AttributeModifier>(), mapIdentifier);
		attributes.add(attribute);
		
		modifiers.add(ClassModifierVisitable.create(VisitableToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierAbstract.create(AbstractToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group=[group2:group=[class1:class={};class2:class={};];]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testParserAfterScanner() throws Exception {
		final IdentifierToken class1Identifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken group1Identifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken class2Identifier = IdentifierToken.create("class2", Position.create("", 1, 1, 0));
		final IdentifierToken group2Identifier = IdentifierToken.create("group2", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput("group1 : group=[group2:group=[class1:class={};class2:class={};];];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<GroupElement> group2Vector = new Vector<>();
		
		final UnqualifiedName group1Name = UnqualifiedName.create(group1Identifier);
		final Group group1 = Group.create(group1Name, group1Vector, GroupToken.create(Position.create("", 1, 1, 0)));
		final Name group2Name = QualifiedName.create(group1Name, UnqualifiedName.create(group2Identifier));
		final Group group2 = Group.create(group2Name, group2Vector, GroupToken.create(Position.create("", 1, 1, 0)));
		group1Vector.add(group2);
		
		final Name class1Name = group2Name.addName(class1Identifier);
		final Name class2Name = group2Name.addName(class2Identifier);
		final ClassType clazz1 =
				RegularClassType.create(
						class1Name,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						class1Identifier,
						
						new Vector<ClassType>());
		final ClassType clazz2 =
				RegularClassType.create(
						class2Name,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						class2Identifier,
						
						new Vector<ClassType>());
		
		group2Vector.add(clazz1);
		group2Vector.add(clazz2);
		final Model expected = Model.create(group1Identifier);
		expected.addGroup(group1);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {} abstract abstract abstract ; ] ; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testMultipleSameClassModifier() throws Exception {
		final SimpleScannerInput input =
				new SimpleScannerInput("group1 : group = [ class1 : class = {} abstract abstract abstract ; ] ; ");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		try {
			parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(2, parser.getExceptions().size());
		}
	}
	
	/**
	 * group1 : group = [ someexception : exception = {};]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSimpleException() throws Exception {
		final IdentifierToken someexceptionIdentifier =
				IdentifierToken.create("someexception", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input = new SimpleScannerInput("group1 : group = [ someexception : exception = {};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		
		final UnqualifiedName groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(someexceptionIdentifier);
		final ClassType clazz =
				ExceptionClassType.create(
						className,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						someexceptionIdentifier,
						new Vector<ClassType>());
		
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {att1 : Integer transient;} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testFindableServiceTransientModifier() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("att1", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {att1 : Integer transient;} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<ClassModifier> classModifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						classModifiers,
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(integerIdentifier, integerByName);
		final Vector<AttributeModifier> attributeModifiers = new Vector<>();
		attributeModifiers.add(AttributeModifierTransient.create(TransientToken.create(Position.create("", 1, 1, 0))));
		
		final Attribute attribute = Attribute.create("att1", typeProxyInteger, attributeModifiers, attributeIdentifier);
		attributes.add(attribute);
		
		classModifiers.add(ClassModifierService.create(ServiceToken.create(Position.create("", 1, 1, 0))));
		classModifiers.add(ClassModifierTransient.create(TransientToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {attribute : [Integer -> [Integer -> [Integer -> Integer*] ] ];} service
	 * transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testSimpleHashMap() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("attribute", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {attribute : [Integer -> [Integer -> [Integer -> Integer*] ] ];} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(integerIdentifier, integerByName);
		
		final ListType list = ListType.create(integerIdentifier, typeProxyInteger);
		final MapType map1 = MapType.create(integerIdentifier, typeProxyInteger, list);
		final MapType map2 = MapType.create(integerIdentifier, typeProxyInteger, map1);
		final MapType map3 = MapType.create(integerIdentifier, typeProxyInteger, map2);
		
		final Attribute attribute =
				Attribute.create("attribute", map3, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		modifiers.add(ClassModifierService.create(ServiceToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(TransientToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
}
