package de.fhdw.wtf.walker.tasks.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.ExecutionException;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.task.TaskExecutor;
import de.fhdw.wtf.common.task.TaskExecutorFixed;
import de.fhdw.wtf.common.task.result.ExceptionalTaskResult;
import de.fhdw.wtf.common.task.result.OKTaskResult;
import de.fhdw.wtf.common.task.result.TaskResult;
import de.fhdw.wtf.common.task.result.visitor.TaskResultVisitor;
import de.fhdw.wtf.walker.tasks.ConstructorReferencer;
import de.fhdw.wtf.walker.tasks.TypeReferencer;

/**
 * Contains all values of the execution of referencer: Collection<OKTaskResult> okResult;
 * Collection<ExceptionalTaskResult> failResult; Collection<TaskResult> results;.
 */
public class ConstructorReferencerTestReturnValue {
	
	/**
	 * Contains all positive results of task execution.
	 */
	private final Collection<OKTaskResult> okResult;
	
	/**
	 * Contains all negative results of task execution.
	 */
	private final Collection<ExceptionalTaskResult> failResult;
	
	/**
	 * The referencer which is executed to get the return values.
	 */
	private final TypeReferencer referencer;
	
	/**
	 * Constructor. Executes the referencing of the given model.
	 * 
	 * @param model
	 *            - model for type referencing.
	 * @throws InterruptedException
	 *             - {@link InterruptedException}
	 * @throws ExecutionException
	 *             - {@link ExecutionException}.
	 */
	public ConstructorReferencerTestReturnValue(final Model model) throws InterruptedException, ExecutionException {
		
		final TaskExecutor taskManager = TaskExecutorFixed.create();
		this.referencer = TypeReferencer.create(model, taskManager);
		taskManager.startAllKnownTasks();
		this.okResult = new ArrayList<>();
		this.failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskManager.getResultsAndShutdown();
		final TaskExecutor taskManager2 = TaskExecutorFixed.create();
		ConstructorReferencer.create(model, taskManager2);
		taskManager2.startAllKnownTasks();
		results.addAll(taskManager2.getResultsAndShutdown());
		
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					ConstructorReferencerTestReturnValue.this.okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					ConstructorReferencerTestReturnValue.this.failResult.add(result);
				}
			});
		}
	}
	
	/**
	 * Returns the collection of positive results of the task execution.
	 * 
	 * @return Collection of positive results.
	 */
	public Collection<OKTaskResult> getOkResult() {
		return this.okResult;
	}
	
	/**
	 * Returns the collection of negative results of the task execution.
	 * 
	 * @return Collection of negative results.
	 */
	public Collection<ExceptionalTaskResult> getFailResult() {
		return this.failResult;
	}
	
	/**
	 * Returns the referencer which is used for referencing the given model.
	 * 
	 * @return used {@link TypeReferencer}.
	 */
	public TypeReferencer getTypeReferencer() {
		return this.referencer;
	}
}
